unit frmResultUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, CommonDataUnit;

type
  TfrmResult = class(TForm)
    ListBox1: TListBox;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    { Private declarations }
    fResultList : TQueryResultList;
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    procedure SetResult(const aData : String);
  end;

var
  frmResult: TfrmResult;

implementation

{$R *.dfm}

uses
  FMCUtilsWrapper
  , SuperObject
  ;

{ TfrmResult }

constructor TfrmResult.Create(AOwner: TComponent);
var
  Icon : TIcon;
begin
  inherited;
  fResultList := TQueryResultList.Create;
  icon := TIcon.Create;
  try
    Icon.Handle := LoadIcon(hInstance, 'ActiveIcon');
    PostMessage(Self.Handle, WM_SETICON, ICON_SMALL, icon.Handle);
  finally
    Icon.free;
  end;
end;

procedure TfrmResult.SetResult(const aData: String);
var
  SuperObj : ISuperObject;
  DetailObj : ISuperObject;
  aRecResult : TRecQueryResult;
  aAESKey : String;
  i : Integer;
  pDetailItem : PRecResultDetail;
  JSONArrIter : TSuperObjectIter;
begin
  fResultList.Clear;
  ListBox1.Items.Clear;
  SuperObj := SO(aData);
  aAESKey := g_FacilityID + 'scn2019x';

  with aRecResult do
  begin
    code := SuperObj['code'].AsString;
    data := AES128DecodeString(SuperObj['data'].AsString, aAESKey);
    name := AES128DecodeString(SuperObj['name'].AsString, aAESKey);

    DetailObj := SO(data);

    for i:=0 to DetailObj.AsArray.Length-1 do
    begin
      if ObjectFindFirst(DetailObj.AsArray.O[i], JSONArrIter) then
      repeat
        New(pDetailItem);
        with pDetailItem^ do
        begin
          Key := JSONArrIter.key;
          Value := JSONArrIter.val.AsString;
        end;
        fResultList.Add(pDetailItem);
        ListBox1.Items.Add('');
      until not ObjectFindNext(JSONArrIter);
      ObjectFindClose(JSONArrIter);
    end;

    Self.Caption := name;
  end;
end;

procedure TfrmResult.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if key = #27 then
    ModalResult := mrOK;
end;

destructor TfrmResult.Destroy;
begin
  FreeAndNil(fResultList);
  inherited;
end;

procedure TfrmResult.ListBox1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  KeyStr, ValueStr : String;
begin
  if Index < 0 then exit;
  if Index > fResultList.Count-1 then exit;

  KeyStr := PRecResultDetail(fResultList.Items[Index])^.Key;
  ValueStr := PRecResultDetail(fResultList.Items[Index])^.Value;

  with TListbox(Control).Canvas do
  begin
    if odSelected in State then
    begin
      Brush.Color := $00238AFF;
      Pen.Color   := $00238AFF;
    end
    else begin
      Brush.Color := clWhite;
      Pen.Color := clWhite;
    end;
    FillRect(Rect);
    Rectangle(Rect);

    Font.Style  := [fsBold];
    Pen.Color   := clBlack;
    Pen.Width   := 1;
    DrawTextEx(Handle, PChar(KeyStr), Length(KeyStr), Rect, DT_VCENTER or DT_SINGLELINE or DT_LEFT, nil);
    DrawTextEx(Handle, PChar(ValueStr), Length(ValueStr), Rect, DT_VCENTER or DT_SINGLELINE or DT_RIGHT, nil);
  end;
end;

end.
